#include "listswin.h"
#include "mpapp.h"
#include "mpdialogs.h"

@implementation MPPlayListsWin

- (id)initWithListsDict:(NSDictionary *)lists;
{
	NSArray *dictKeys;
	NSRect myRect = {{200, 200}, {150, 200}};
	NSRect matrixRect = {{0, 0}, {150, 200}};
	unsigned int style = NSTitledWindowMask | NSClosableWindowMask				
							| NSMiniaturizableWindowMask | NSResizableWindowMask;
	id cell;
	int i;
	
	self = [super initWithContentRect:myRect
									styleMask:style
									  backing:NSBackingStoreRetained
										 defer:NO];
  	if(self) {
		[self setTitle:@"Lists"];
		[self setFrameAutosaveName:@"Lists"]; 
		
		listsDict = [[NSMutableDictionary alloc] initWithDictionary: lists];
		listWins = [[NSMutableArray alloc] initWithCapacity: 1];

		myView = [[NSView alloc] initWithFrame: frame];
		[self setContentView: myView];

  		buttonCell = [[NSButtonCell new] autorelease];
  		[buttonCell setButtonType: NSPushOnPushOffButton];
		[buttonCell setBordered:NO];
		[buttonCell setAlignment:NSLeftTextAlignment];	

  		listsMatrix = [[NSMatrix alloc] initWithFrame:matrixRect
				       								 	  mode:NSRadioModeMatrix
				  								  		prototype:buttonCell
			       						  		numberOfRows:0
			    						  		numberOfColumns:0];

		[listsMatrix setAutoresizingMask: NSViewWidthSizable];
		[listsMatrix setTarget: self];
		[listsMatrix setDoubleAction: @selector(handleListsMatrixDoubleClick:)];
											
 		listsScroll = [[NSScrollView alloc] initWithFrame: matrixRect];
		[listsScroll setAutoresizingMask: NSViewHeightSizable | NSViewWidthSizable];
  		[listsScroll setHasVerticalScroller: YES];
  		[listsScroll setHasHorizontalScroller: NO];
		
		dictKeys = [listsDict allKeys];
		for(i = 0; i < [dictKeys count]; i++) {
 			[listsMatrix addRow];
  			cell = [listsMatrix cellAtRow:i column:0];
			[cell setTitle: [dictKeys objectAtIndex:i]];
		}
  		[listsScroll setDocumentView: listsMatrix];
  		[myView addSubview: listsScroll];

   	[[NSNotificationCenter defaultCenter] addObserver:self 
                					selector:@selector(addListDialogReturn:) 
                						 name:@"MPNewSongListNotification"
                					  object:nil];
	}
	return self;
}

- (void) dealloc
{
	[listWins removeAllObjects];
	[listWins release];
	[listsDict release];
	[listsMatrix release];
	[listsScroll release];
  	[super dealloc];
}

- (BOOL)performKeyEquivalent:(NSEvent *)theEvent
{
	NSString *commchar;
	
	if(self == (MPPlayListsWin *)[NSApp keyWindow]) {
		commchar = [theEvent charactersIgnoringModifiers];

		if([commchar isEqualToString: @"q"])
			[NSApp terminate: self];
		if([commchar isEqualToString: @"n"])
			[self addList];
		if([commchar isEqualToString: @"o"])
			[self openList];
		if([commchar isEqualToString: @"d"])
			[self removeList];
		if([commchar isEqualToString: @"w"])
			[self close];

		return YES;
	}
	return NO;
}

- (NSDictionary *)listsDict
{
	return listsDict;
}

- (void)handleListsMatrixDoubleClick:(id)sender
{
	[self openList];
}

- (void)addList
{
	NewSongListDlg *dialog;
	
	dialog = [[NewSongListDlg alloc] init];
	[dialog display];
	[dialog orderFront:nil];
}

- (void)addListDialogReturn:(NSNotification *)aNotification
{
	NewSongListDlg *dialog;
	NSArray *dictKeys;
	NSMutableArray *songsArr;
	NSString *newListName;
	id cell;
	BOOL found = NO;
	int i;

	dialog = (NewSongListDlg *)[aNotification object];
	if([dialog getresult]) {
		newListName = [dialog getFirstValue];
		dictKeys = [listsDict allKeys];
		for(i = 0; i < [dictKeys count]; i++)
			if([[dictKeys objectAtIndex:i] isEqualToString: newListName]) 
				found = YES;
		if(found) {
			[dialog close];
			[self runAlert:@"This list is already present!"];
			return;
		} else {

			[listsMatrix addRow];
			cell = [listsMatrix cellAtRow:[listsMatrix numberOfRows]-1 column:0];
			[cell setTitle: newListName];
			[cell setTarget: self];
			[cell setAction: @selector(handleListsMatrixSelect:)];
  			[listsMatrix setNeedsDisplay: YES];
			[listsMatrix scrollCellToVisibleAtRow:[listsMatrix numberOfRows]-1 column:0];
			[listsMatrix selectCellAtRow:[listsMatrix numberOfRows]-1 column:0]; 

			songsArr = [NSMutableArray arrayWithCapacity: 1];
			[listsDict setObject: songsArr forKey: newListName];
			[[MPApp sharedmpapp] saveChangesToPlayListsDict: nil];
			
			[dialog close];
		}
	} else {
		[dialog close];
	}
}

- (void)removeList
{
	NSString *cellTxt;
	NSArray *dictKeys;
	MPPlaySongsWin	*songsWin;
	id cell;
	float x, y;
	int i, result;
	
	if(![self isVisible])
		return;

	cellTxt = [[listsMatrix selectedCell] title];
	result = NSRunAlertPanel(@"delete list", 
[NSString stringWithFormat:@"Are you sure you want to delete
\"%@\"?", cellTxt], @"OK", @"Cancel", NULL);
	if(result != NSAlertDefaultReturn)
		return;

	[listsDict removeObjectForKey:cellTxt];
	x = [listsMatrix frame].size.width;
	y = [listsMatrix frame].size.height;
	
	[listsMatrix release];
  	listsMatrix = [[NSMatrix alloc] initWithFrame:NSMakeRect(0, 0, x, y)
				       								  mode:NSRadioModeMatrix
				  								  	prototype:buttonCell
			       						  	numberOfRows:0
			    						  	numberOfColumns:0];
	[listsMatrix setAutoresizingMask: NSViewWidthSizable];
	dictKeys = [listsDict allKeys];
	for(i = 0; i < [dictKeys count]; i++) {
 		[listsMatrix addRow];
  		cell = [listsMatrix cellAtRow:i column:0];
		[cell setTitle: [dictKeys objectAtIndex:i]];
		[cell setTarget: self];
		[cell setAction: @selector(handleListsMatrixSelect:)];
	}
  	[listsScroll setDocumentView: listsMatrix];
  	[listsMatrix setNeedsDisplay: YES];
		
	for(i = 0; i < [listWins count]; i++) {
		songsWin = [listWins objectAtIndex: i];
		if([[songsWin getTitle] isEqualToString: cellTxt]) {
			[songsWin close];
			[listWins removeObject: songsWin];
			break;
		}
	}

	[[MPApp sharedmpapp] saveChangesToPlayListsDict: nil];
}

- (void)openList
{
	NSString *cellTxt;
	NSArray *songsNames;
	MPPlaySongsWin	*songsWin;
	int i;

	if((![listsDict count]) || (![self isVisible])) 
		return;

	cellTxt = [[listsMatrix selectedCell] title];	
	songsNames = [listsDict objectForKey: cellTxt];		
	
	for(i = 0; i < [listWins count]; i++) {
		songsWin = [listWins objectAtIndex: i];
		if([[songsWin getTitle] isEqualToString: cellTxt]) {
			[songsWin display];
			[songsWin orderFront:nil];
			return;
		}
	}
	songsWin = [[MPPlaySongsWin alloc] initWithSongs:songsNames title:cellTxt listsWin:self];
	[songsWin setReleasedWhenClosed: NO];
	[songsWin display];
	[songsWin orderFront:nil];
	[listWins addObject: songsWin];
	[songsWin release];
}

- (void)saveChangesInSongsWin:(MPPlaySongsWin *)aWin
{
	[listsDict removeObjectForKey: [aWin getTitle]];
	[listsDict setObject: [aWin songsArr] forKey: [aWin getTitle]];
	[[MPApp sharedmpapp] saveChangesToPlayListsDict: nil];
}

- (void)handleDoubleClick:(id)sender
{
	printf("AAAAAAAAAAAAAAAAAAAAAAAAAA\n");
}

- (void)runAlert:(NSString *)error
{
	NSRunAlertPanel(nil, [NSString stringWithString:error], @"Continue", nil, nil);
}

@end
