#import "mpapp.h"
#include "bindirs.h"

static MPApp *sharedmpapp = nil;
BOOL isplaing;
MPControls *controls;
MPDisplay *display;

int catchXError(Display *dpy, XErrorEvent *error)
{
	return 0;
}

@implementation MPApp

+ (MPApp *)sharedmpapp
{
	if(!sharedmpapp) {
      NS_DURING
      {
	 	 	[gnustep_global_lock lock];
	  		sharedmpapp = [[self alloc] init];
	  		[gnustep_global_lock unlock];
		}
		NS_HANDLER
		{
	  		[gnustep_global_lock unlock];
	  		[localException raise];
		}
      NS_ENDHANDLER
	}
	return sharedmpapp;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	NSArray *r;
	NSArray *s;
	NSView *wview;
	NSRect winRect;
	NSRect controlsRect = {{0, 0}, {218, 36}};
	NSRect displayRect = {{0, 37}, {218, 65}};
	unsigned int style = NSTitledWindowMask | NSClosableWindowMask	| NSMiniaturizableWindowMask;

	XSetErrorHandler((XErrorHandler)catchXError);	

	r = [NSArray arrayWithObjects: NSStringPboardType, nil];
	s = [NSArray arrayWithObjects: NSStringPboardType, nil];
	[[NSApplication sharedApplication] registerServicesMenuSendTypes: s returnTypes: r];
	
	defaults = [NSUserDefaults standardUserDefaults];
  	pInfo = [NSProcessInfo processInfo];
  	env = [pInfo environment];
	fm = [NSFileManager defaultManager];

	playListsDict = (NSMutableDictionary *)[[defaults dictionaryForKey:@"lists"] retain];
	if(!playListsDict) {
		playListsDict = [[NSMutableDictionary alloc] initWithCapacity:1];
		[defaults setObject:playListsDict forKey:@"lists"];
	}
	[defaults synchronize];
	
	currSongList = [[NSArray array] retain];
	currSongPath = [[NSString string] retain];

	player = [MPPlayer player];
	[player initialize];
	isplaing = NO;
	listplaing = NO;
	listpos = -1;
	
	winRect = NSMakeRect(400, 320, 218, 102);
	
	win = [[NSWindow alloc] initWithContentRect:winRect
												 styleMask:style
													backing:NSBackingStoreRetained
													  defer:NO];
	[win setTitle:@"MPlay.app"];
	[win setFrameAutosaveName:@"MPlay.app"]; 
	wview = [win contentView];
		
	controls = [[MPControls alloc] initWithFrame:controlsRect];
	[wview addSubview: controls];
	display = [[MPDisplay alloc] initWithFrame:displayRect ofWindow:win];
	[wview addSubview: display];	
	
	[win display];
	[win orderFront:nil];

	listsWin = [[MPPlayListsWin alloc] initWithListsDict: playListsDict];
	[listsWin setReleasedWhenClosed: NO];
}

- (BOOL)applicationShouldTerminate:(NSApplication *)app 
{
	close(1);

	return YES;
}

- (BOOL)application:(NSApplication *)app openFile:(NSString *)filename
{
	if(currSongPath)
		[currSongPath release];
	currSongPath = [[NSString alloc] initWithString: filename];
	[self play];
	return YES;
}

- (void)method:menuCell
{

}

- (void)saveChangesToPlayListsDict:(id)sender
{
	[playListsDict release];
	playListsDict = [[NSMutableDictionary alloc] initWithDictionary: [listsWin listsDict]];
	[defaults setObject:playListsDict forKey:@"lists"];
	[defaults synchronize];
}

- (void)setNewSongToPlay:(NSString *)songPath
{
	[currSongPath release];
	currSongPath = [[NSString alloc] initWithString: songPath];
	if(isplaing)
		[player stop];
	listplaing = NO;
	listpos = -1;	
}

- (void)setNewListToPlay:(NSArray *)listToPlay
{
	[currSongList release];
	currSongList = [[NSArray alloc] initWithArray: listToPlay];
	if(isplaing)
		[player stop];
	listpos = -1;	
	listplaing = YES;
}

- (NSString *)currSongPath
{
	return currSongPath;
}

- (void)play
{
	if(![currSongPath length] && !listplaing) {
		NSRunAlertPanel(nil, @"You must create a list and add a song!", @"Continue", nil, nil);
		[controls resetButtons];
		return;
	}
	
	if(listplaing) {
		listpos++;
		if(listpos >= [currSongList count])
			listpos = 0;
		[display setTrackField: listpos + 1];
		[[MPPlayer player] play: [currSongList objectAtIndex: listpos]];
	} else {
		[display setTrackField: 1];
		[[MPPlayer player] play: currSongPath];
	}
	isplaing = YES;
}

- (void)stop
{
	[[MPPlayer player] stop];
	isplaing = NO;
	
	if(listplaing)
		listpos--;
}

- (void)nextSong
{
	if(listplaing) {
		[[MPPlayer player] stop];
		isplaing = NO;
		[controls setPlayStopButtonPos: NO];
		[self play];
	}
}

- (void)previousSong
{
	if(listplaing) {
		if(listpos >= 1) {
			listpos = listpos - 2;
			[[MPPlayer player] stop];
			isplaing = NO;
			[controls setPlayStopButtonPos: NO];
			[self play];
		}
	}
}

- (NSString *)fullPathForApplication:(NSString *)appName
{
	NSString *fullAppPath;
	NSArray *paths;
	int i;
	BOOL exists, isdir;
	
	paths = [bindirs componentsSeparatedByString:@"\n"];
	for(i = 0; i < [paths count]; i++) {
		fullAppPath = [[paths objectAtIndex:i] stringByAppendingPathComponent:appName];
		exists = [fm fileExistsAtPath:fullAppPath isDirectory:&isdir];
		if(exists && !isdir) {
			if([fm isExecutableFileAtPath:fullAppPath])
				return fullAppPath;
		}
	}
	return nil;
}

@end
