#include "mpdisplay.h"
#include "mpapp.h"

int t_sampling_frequency[2][3] = { 
	{22050, 24000, 16000}, 
	{44100, 48000, 32000}
};

extern BOOL isplaing;
NSRect trackLabRect = {{10, 45}, {50, 15}};
NSRect timeLabRect = {{80, 45}, {50, 15}};
NSRect frameLabRect = {{150, 45}, {50, 15}};
NSRect trackFieldRect = {{5, 20}, {55, 25}};
NSRect timeFieldRect = {{70, 20}, {70, 25}};
NSRect framefieldRect = {{158, 30}, {34, 15}};
NSRect songFieldRect = {{10, 5}, {205, 20}};	

@implementation MPDisplay

- (id)initWithFrame:(NSRect)frameRect ofWindow:(NSWindow *)win
{	
	NSFont *font;

	self = [super initWithFrame:frameRect];
  	if(self) {
		myWin = win;
		font = [NSFont systemFontOfSize: 10];

		trackLabel = [[[NSTextField alloc] initWithFrame:trackLabRect] autorelease];
		[trackLabel setFont:font];
		[trackLabel setAlignment:NSCenterTextAlignment];	
		[trackLabel setBackgroundColor:[NSColor blackColor]];
		[trackLabel setBezeled:NO];
		[trackLabel setEditable:NO];
		[trackLabel setSelectable:NO];
		[trackLabel setTextColor:[NSColor greenColor]];
		[trackLabel setStringValue:@"TRACK"];
		[self addSubview: trackLabel];

		timeLabel = [[[NSTextField alloc] initWithFrame:timeLabRect] autorelease];
		[timeLabel setFont:font];
		[timeLabel setAlignment:NSCenterTextAlignment];	
		[timeLabel setBackgroundColor:[NSColor blackColor]];
		[timeLabel setBezeled:NO];
		[timeLabel setEditable:NO];
		[timeLabel setSelectable:NO];
		[timeLabel setTextColor:[NSColor greenColor]];
		[timeLabel setStringValue:@"TIME"];
		[self addSubview: timeLabel];

		frameLabel = [[[NSTextField alloc] initWithFrame:frameLabRect] autorelease];
		[frameLabel setFont:font];
		[frameLabel setAlignment:NSCenterTextAlignment];	
		[frameLabel setBackgroundColor:[NSColor blackColor]];
		[frameLabel setBezeled:NO];
		[frameLabel setEditable:NO];
		[frameLabel setSelectable:NO];
		[frameLabel setTextColor:[NSColor greenColor]];
		[frameLabel setStringValue:@"FRAME"];
		[self addSubview: frameLabel];

		font = [NSFont systemFontOfSize: 18];
		
		trackField = [[[NSTextField alloc] initWithFrame:trackFieldRect] autorelease];
		[trackField setFont:font];
		[trackField setAlignment:NSCenterTextAlignment];	
		[trackField setBackgroundColor:[NSColor blackColor]];
		[trackField setBezeled:NO];
		[trackField setEditable:NO];
		[trackField setSelectable:NO];
		[trackField setTextColor:[NSColor greenColor]];
		[trackField setIntValue: 0];
		[self addSubview: trackField];

		timeField = [[[NSTextField alloc] initWithFrame:timeFieldRect] autorelease];
		[timeField setFont:font];
		[timeField setAlignment:NSCenterTextAlignment];	
		[timeField setBackgroundColor:[NSColor blackColor]];
		[timeField setBezeled:NO];
		[timeField setEditable:NO];
		[timeField setSelectable:NO];
		[timeField setTextColor:[NSColor greenColor]];
		[timeField setStringValue:@"00:00"];
		[self addSubview: timeField];

		font = [NSFont systemFontOfSize: 10];

		framefield = [[[NSTextField alloc] initWithFrame:framefieldRect] autorelease];
		[framefield setFont:font];
		[framefield setAlignment:NSCenterTextAlignment];	
		[framefield setBackgroundColor:[NSColor blackColor]];
		[framefield setBezeled:NO];
		[framefield setEditable:NO];
		[framefield setSelectable:NO];
		[framefield setTextColor:[NSColor greenColor]];
		[framefield setIntValue: 0];
		[self addSubview: framefield];

		songField = [[[NSTextField alloc] initWithFrame:songFieldRect] autorelease];
		[songField setFont:font];
		[songField setAlignment:NSLeftTextAlignment];	
		[songField setBackgroundColor:[NSColor blackColor]];
		[songField setBezeled:NO];
		[songField setEditable:NO];
		[songField setSelectable:NO];
		[songField setTextColor:[NSColor orangeColor]];
		[songField setStringValue:@"Song:"];
		[self addSubview: songField];

		currFrame = 0;
		minutes = 0;
		seconds = 0;
	}
  	return self;	
}

- (void) dealloc
{
  	[super dealloc];
}

- (void)clearDisplay
{
	if(ttimer)
		if([ttimer isValid])
			[ttimer invalidate];
	currFrame = 0;
	minutes = 0;
	seconds = 0;

	[trackField setIntValue: 0];
	[self setNeedsDisplayInRect: trackFieldRect];
	[timeField setStringValue:@"00:00"];
	[self setNeedsDisplayInRect: timeFieldRect];
	[framefield setIntValue: 0];
	[self setNeedsDisplayInRect: framefieldRect];
	[songField setStringValue:@"Song:"];
	[self setNeedsDisplayInRect: songFieldRect];
	[myWin update];
}

- (void)setTrackField:(int)track
{
	[trackField setIntValue: track];
	[self setNeedsDisplayInRect: trackFieldRect];
	[myWin update];
}

- (void)setTimeField:(NSString *)timeStr
{
	[timeField setStringValue: timeStr];
	[self setNeedsDisplay: YES];
	[myWin update];
}

- (void)setFrameField:(int)frameNum
{
	currFrame = frameNum;
	[framefield setIntValue: currFrame];
	[self setNeedsDisplayInRect: framefieldRect];
	[myWin update];
}

- (void)setSongField:(NSString *)songName
{
	[songField setStringValue: songName];
	[self setNeedsDisplay: YES];
	[myWin update];
}

- (void)updateAudioInfo:(AudioInfo)ai
{	
	[myWin update];
	ttimer = [NSTimer scheduledTimerWithTimeInterval:1 target:self 
							selector:@selector(timestep:) userInfo:nil repeats:YES];
}

- (void)timestep:(NSTimer *)t
{
	NSString *timeStr;

	seconds++;
	if(seconds == 60) {
		seconds = 0;
		minutes++;
	}
	timeStr = [NSString stringWithFormat:@"%d:%02d", minutes, seconds];
	[self setTimeField: timeStr];
}

- (void)drawRect:(NSRect)rect
{
	PSsetgray(NSBlack);
	NSRectFill(rect);

  	[[NSColor lightGrayColor] set];
  	PSmoveto(0, 0);
 	PSlineto(0, 65);
	PSstroke();

  	[[NSColor whiteColor] set];
  	PSmoveto(216, 0);
 	PSlineto(216, 65);
	PSstroke();
 	[[NSColor lightGrayColor] set];
  	PSmoveto(217, 0);
 	PSlineto(217, 65);
	PSstroke();
  	PSmoveto(218, 0);
 	PSlineto(218, 65);
	PSstroke();

 	[[NSColor lightGrayColor] set];
  	PSmoveto(0, 65);
 	PSlineto(218, 65);
	PSstroke();

}

@end
