#import "mpplayer.h"
#import "mpapp.h"
#import "mpcontrols.h"
#import "mpdisplay.h"

#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <limits.h>
#include <sys/uio.h>
#include <sys/un.h>
#include <errno.h>
#include <math.h>
#include <sysexits.h>

#if !defined(SCM_RIGHTS)
	#define SCM_RIGHTS 1
#endif

#ifdef EX__MAX
	#define FORK_ERROR EX__MAX + 1
#else 
	#define FORK_ERROR 80
#endif

#define MSGIN receive[0]
#define MSGOUT send[1]

extern MPControls *controls;
extern MPDisplay *display;

static MPPlayer *player = nil;

static void songkill(int);

static void initsigchild()
{
	signal(SIGCHLD, songkill);
}

static void songkill(int sn)
{
	int status;
	BOOL dn;
	
   if(waitpid(-1, &status, WNOHANG) == [player fork_pid]) {
     	dn = YES;
      if(WIFEXITED(status) && WEXITSTATUS(status) == FORK_ERROR) {
         [player setdontFork:YES];
   		[[NSNotificationCenter defaultCenter] 
						postNotificationName:@"MPSongKilledNotification" object:nil]; 
         dn = NO;
      }
		[player childDied:dn];
   }
   initsigchild();
}

@implementation MPPlayer

+ (MPPlayer *)player
{
	if(!player) {
      NS_DURING
      {
	 	 	[gnustep_global_lock lock];
	  		player = [[self alloc] init];
	  		[gnustep_global_lock unlock];
		}
		NS_HANDLER
		{
	  		[gnustep_global_lock unlock];
	  		[localException raise];
		}
      NS_ENDHANDLER
	}
	return player;
}

- (void)initialize
{
   dontFork = NO;
   isForwarding = NO;
   isRewinding = NO;	
	forwrewCount = 0;
   fork_pid = -1;

   initsigchild();
   [[NSNotificationCenter defaultCenter] addObserver:self 
            						selector:@selector(songKilled:)
								name:@"MPSongKilledNotification" object:nil];
	[self forkIt];						
}

- (int)fork_pid 
{
	return fork_pid;
}

- (void)setdontFork:(BOOL)boolval
{
	dontFork = boolval;
}

- (void)songKilled:(NSNotification *)aNotification
{
	dontFork = NO;
}

- (void)forkIt
{
	int i;
	
	if(dontFork)
   	return;

   signal(SIGPIPE, SIG_IGN);

   if(socketpair(AF_UNIX, SOCK_STREAM, 0, send) < 0)
      perror("socketpair");
   if(socketpair(AF_UNIX, SOCK_STREAM, 0, receive) < 0)
      perror("socketpair");

   if((fork_pid = fork()) == 0) {
      dup2(send[0], STDIN_FILENO);
      close(send[0]);
      close(send[1]);
      dup2(receive[1], STDOUT_FILENO);
      close(receive[0]);
      close(receive[1]);
		
		execlp("amp", "-g", NULL);		
		
      for(i = 0; i < OPEN_MAX; i++)
      	close(i);

      exit(FORK_ERROR);
   }

	close(send[0]);
	close(receive[1]);

	readSocketHandle = [[NSFileHandle alloc] initWithFileDescriptor:receive[0]]; 

	[[NSNotificationCenter defaultCenter] addObserver:self
							 selector:@selector(pendingmsg:)
							     name:NSFileHandleDataAvailableNotification
							   object:(id)readSocketHandle];
	[readSocketHandle waitForDataInBackgroundAndNotify];
}

- (void)parse_msg:(PControlMsg)msg
{
	AudioInfo ai;
	int err;
	double percent;

   switch(msg->type) {
	
      case MSG_NEXT:
         [self next];
         break;
			
      case MSG_INFO: 
		{
		
			printf("MSG_INFO\n");
			
         while((err = read(MSGIN, &ai, sizeof(AudioInfo))) < 0) {
            if(err == EINTR)
               continue;
            break;
         }
			[display updateAudioInfo: ai];
      }
      	break;
			
      case MSG_FRAMES:
			[display setFrameField: msg->data];
         break;
			
      case MSG_AUDIOFAILURE:
			printf("mplay: MSG_AUDIOFAILURE\n");
  //       QMessage::messageDismiss("Audiofailure", JC_ERROR);
         break;
			
      case MSG_POSITION:
	//		printf("mplay: MSG_POSITION\n");    // QUESTO FUNZIONA !
         if(length <= 0) {
     //       emit songPosition(100.0);
         } else {
            percent = (double)msg->data / (double)length;

    //        emit songPosition(percent);
         }
         break;
			
      case MSG_RESPONSE:
         switch(msg->data) {
            case FORWARD_BEGIN:
					[self forwardStep];
					break;
            case FORWARD_STEP:
					[self forwardStep];
					break;
	         case FORWARD_END:
					[self forwardEnd];
					break;					
					
            case REWIND_BEGIN:
					[self rewindStep];
					break;
            case REWIND_STEP:
					[self rewindStep];
					break;
           	case REWIND_END:
					[self rewindEnd];
					break;
         }
         break;
    }
}

- (void)pendingmsg:(NSNotification *)notification
{
   TControlMsg msg;

   read(receive[0], &msg, sizeof(TControlMsg));
	[self parse_msg:&msg];
	[readSocketHandle waitForDataInBackgroundAndNotify];
}

- (void)childDied:(BOOL)dontNotify
{
   if(fork_pid != -1) {
      fork_pid = -1;
      
		[readSocketHandle release];
     	close(MSGOUT);
    	close(MSGIN);

      if(!dontNotify) {
  //       QMessage::messageDismiss(
 //           "Execution of the player failed. Make sure "
 //           "you have speicified the correct path and "
 //           "that the permissions are correctly set.",
 //           JC_ERROR);
      } else {
         [self next];
		}
   }
}

- (void)send_msg:(PControlMsg)msg
{
   if(dontFork)
      return;
	
   if(fork_pid == -1)
   	[self forkIt];

   write(MSGOUT, msg, sizeof(TControlMsg));
}

- (void)get_msg:(PControlMsg)msg
{
   read(MSGIN, msg, sizeof(TControlMsg));
}

- (void)next
{
	[display clearDisplay];
	[[MPApp sharedmpapp] nextSong];
}

- (void)forward
{
   TControlMsg msg;
	
	if((!isForwarding) && (!isRewinding)) {
		forwrewCount = 0;
		isForwarding = YES;
      msg.type = MSG_CTRL;
      msg.data = FORWARD_BEGIN;
      [self send_msg:&msg];
	}
}

- (void)forwardStep
{
   TControlMsg msg;

	msg.type = MSG_CTRL;
	if(forwrewCount < 10) {
		forwrewCount++;
      msg.data = FORWARD_STEP;
	} else {
		msg.data = FORWARD_END;
	}
   [self send_msg:&msg];
}

- (void)forwardEnd
{
   isForwarding = NO;
	forwrewCount = 0;
}

- (void)rewind
{
   TControlMsg msg;

	if((!isRewinding) && (!isForwarding)) {
		forwrewCount = 0;
		isRewinding = YES;
      msg.type = MSG_CTRL;
      msg.data = REWIND_BEGIN;
      [self send_msg:&msg];
	}
}

- (void)rewindStep
{
   TControlMsg msg;

	msg.type = MSG_CTRL;
	if(forwrewCount < 10) {
		forwrewCount++;
      msg.data = REWIND_STEP;
	} else {
		msg.data = REWIND_END;
	}
   [self send_msg:&msg];
}

- (void)rewindEnd
{
   isRewinding = NO;	
	forwrewCount = 0;
}

- (void)stop
{
   TControlMsg msg;

   if(fork_pid == -1)
      return;
   msg.type = MSG_CTRL;
   msg.data = PLAY_STOP;
   [self send_msg:&msg];
	
	[controls resetButtons];
	[display clearDisplay];
}

- (BOOL)sendQuery:(PControlMsg)msg
{
   TControlMsg rmsg;
   
	[readSocketHandle release];
	
   [self send_msg: msg];
   for(;;) {
      [self get_msg:&rmsg];
      if(rmsg.type != MSG_RESPONSE)
         [self parse_msg:&rmsg];
      else
         break;
   }   

	readSocketHandle = [[NSFileHandle alloc] initWithFileDescriptor:receive[0]]; 

	[[NSNotificationCenter defaultCenter] addObserver:self
							 selector:@selector(pendingmsg:)
							     name:NSFileHandleDataAvailableNotification
							   object:(id)readSocketHandle];
	[readSocketHandle waitForDataInBackgroundAndNotify];

   return rmsg.data;
}

- (BOOL)isPlaying
{
   TControlMsg msg;

   msg.type = MSG_QUERY;
   msg.data = QUERY_PLAYING;

   return [self sendQuery:&msg];
}

- (void)play:(NSString *)songPath
{
   int fd;
 	TControlMsg msg;
   struct msghdr hdr;
   struct m_cmsghdr fdhdr;
   struct iovec iov[1];
   char data[2];

	fd = open([songPath cString], O_RDONLY);
	if(fd > 0) {
		[display setSongField: [songPath lastPathComponent]];

		msg.type = MSG_SONG;
		msg.data = 0;
         
		[self send_msg:&msg];

		iov[0].iov_base = data;
		iov[0].iov_len = 2;

		hdr.msg_iov = iov;
		hdr.msg_iovlen = 1; 
		hdr.msg_name = NULL;
	 	hdr.msg_namelen = 0;
		hdr.msg_flags = 0;
         
		fdhdr.cmsg_len = sizeof(struct m_cmsghdr);
		fdhdr.cmsg_level = SOL_SOCKET;
		fdhdr.cmsg_type = SCM_RIGHTS;
		fdhdr.fd = fd;

		hdr.msg_control = &fdhdr;

		hdr.msg_controllen = sizeof(struct m_cmsghdr);

		if(sendmsg(MSGOUT, &hdr, 0) < 0)
			perror("sendmsg");

		close(fd);
	} else {
		msg.type = MSG_RELEASE;
 //     msg.data = configuration.player.release;
      
		[self send_msg:&msg];
	}
}

@end
