#include "songswin.h"
#include "mpapp.h"
#include "mpdialogs.h"
#include "listswin.h"

@implementation ButtonsFrame

- (id)initInWindow:(MPPlaySongsWin *)win rect:(NSRect)rect
{
	NSRect buttRect;
	NSImage *icn;
	NSString *iconpath;
	
	self = [super initWithFrame: rect];
  	if(self) {
		myWin = win;

		buttRect = NSMakeRect(5, 5, 50, 50);
	  	addSongButt = [[NSButton alloc] initWithFrame: buttRect];
	  	[addSongButt setButtonType: NSMomentaryLight];
		iconpath = [[[NSBundle mainBundle] bundlePath]
						stringByAppendingString:@"/Resources/M3Play_add_song.tiff"];
		icn = [[NSImage alloc] initWithContentsOfFile: iconpath];
		[addSongButt setImage: icn];
		[addSongButt setImagePosition: NSImageOnly];
		[icn release];
		[addSongButt setTarget: myWin];
		[addSongButt setAction: @selector(addSong:)];
		[self addSubview: addSongButt]; 

		buttRect = NSMakeRect(60, 5, 50, 50);
	  	remSongButt = [[NSButton alloc] initWithFrame: buttRect];
	  	[remSongButt setButtonType: NSMomentaryLight];
		iconpath = [[[NSBundle mainBundle] bundlePath]
						stringByAppendingString:@"/Resources/M3Play_delete_song.tiff"];
		icn = [[NSImage alloc] initWithContentsOfFile: iconpath];
		[remSongButt setImage: icn];
		[remSongButt setImagePosition: NSImageOnly];
		[icn release];
		[remSongButt setTarget: myWin];
		[remSongButt setAction: @selector(removeSong:)];
		[self addSubview: remSongButt]; 

		buttRect = NSMakeRect(115, 5, 50, 50);
	  	playSongButt = [[NSButton alloc] initWithFrame: buttRect];
	  	[playSongButt setButtonType: NSMomentaryLight];
		iconpath = [[[NSBundle mainBundle] bundlePath]
						stringByAppendingString:@"/Resources/M3Play_play_song.tiff"];
		icn = [[NSImage alloc] initWithContentsOfFile: iconpath];
		[playSongButt setImage: icn];
		[playSongButt setImagePosition: NSImageOnly];
		[icn release];
		[playSongButt setTarget: myWin];
		[playSongButt setAction: @selector(playSong:)];
		[self addSubview: playSongButt]; 

		buttRect = NSMakeRect(170, 5, 50, 50);
	  	playListButt = [[NSButton alloc] initWithFrame: buttRect];
	  	[playListButt setButtonType: NSMomentaryLight];
		iconpath = [[[NSBundle mainBundle] bundlePath]
						stringByAppendingString:@"/Resources/M3Play_play_list.tiff"];
		icn = [[NSImage alloc] initWithContentsOfFile: iconpath];
		[playListButt setImage: icn];
		[playListButt setImagePosition: NSImageOnly];
		[icn release];
		[playListButt setTarget: myWin];
		[playListButt setAction: @selector(playList:)];
		[self addSubview: playListButt]; 
	}
	return self;
}

- (void)dealloc
{
	[addSongButt release];
	[remSongButt release];
	[playSongButt release];
	[playListButt release];
}

@end

@implementation MPPlaySongsMatrix

- (id)initWithFrame:(NSRect)frameRect
	       		mode:(int)aMode
	  		 prototype:(NSCell *)aCell
       numberOfRows:(int)rowsHigh
    numberOfColumns:(int)colsWide
	 		  inWindow:(MPPlaySongsWin *)win
{
	self = [super initWithFrame:frameRect
	       					  mode:aMode
	  		 				prototype:aCell
       				numberOfRows:rowsHigh
    				numberOfColumns:colsWide];
  	if(self) {
		myWin = win;
		isDragTarget = NO;
		
   	[[NSDistributedNotificationCenter defaultCenter] addObserver:self 
                					selector:@selector(fileSystemDidChange:) 
                						 name:@"GFFileSystemDidChangeNotification" 
                					  object:nil];

		[self registerForDraggedTypes: [NSArray arrayWithObject: @"FileObj"]];
	}
	return self;
}

- (void)fileSystemDidChange:(NSNotification *)aNotification
{
	printf("fileSystemDidChange!\n");
}

- (void)drawRect:(NSRect)rect
{
	NSRect highRect;

	[super drawRect: rect];
	highRect = [(NSClipView *)[self superview] documentVisibleRect];
	if(isDragTarget)
  		[[NSColor cyanColor] set];
	else
  		[[NSColor lightGrayColor] set];
	NSFrameRect(highRect);
}

@end

@implementation MPPlaySongsMatrix (DraggingDestination)

- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pb;

	pb = [sender draggingPasteboard];
  	if([[pb types] indexOfObject:@"FileObj"] != NSNotFound) {
		isDragTarget = YES;	
		[self setNeedsDisplay: YES];
		
		printf("draggingEntered\n");
		
    	return NSDragOperationAll;
  	}
	isDragTarget = NO;	
  	return NSDragOperationNone;
}

- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
  	return NSDragOperationAll;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
	isDragTarget = NO;
	[self setNeedsDisplay: YES];
	
	printf("draggingExited\n");
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{

	printf("prepareForDragOperation\n");
	
	return isDragTarget;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{

	printf("performDragOperation\n");
	
	return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard *pb;
	NSData *pbData;
	NSArray *sourcePaths;
	NSString *songPath;

	printf("concludeDragOperation\n");

  	pb = [sender draggingPasteboard];
  	pbData = [pb dataForType: @"FileObj"];
	if(!pbData)
		return;
	sourcePaths = (NSArray *)[NSUnarchiver unarchiveObjectWithData: pbData];  

			// QUA !!  ^^^^^^^^^^^^^^^^^^^^^^^^^
			
	if([sourcePaths count] > 1) {
		NSRunAlertPanel(nil, @"You can add only one song!", @"Continue", nil, nil);
		isDragTarget = NO;
		return;
	}
	songPath = [sourcePaths objectAtIndex: 0];
	[myWin addSongFromDnD: songPath];
	isDragTarget = NO;
	[self setNeedsDisplayInRect: [self frame]];
}

@end


@implementation MPPlaySongsWin

- (id)initWithSongs:(NSArray *)songs 
				  title:(NSString *)atitle 
			  listsWin:(MPPlayListsWin *)awin
{
	NSRect myRect = {{0, 0}, {225, 300}};
	NSRect scrollrect, buttsrect;
	unsigned int style = NSTitledWindowMask | NSClosableWindowMask	| NSMiniaturizableWindowMask;
	id cell;
	int i;
	
	self = [super initWithContentRect:myRect
									styleMask:style
									  backing:NSBackingStoreRetained
										 defer:NO];
  	if(self) {
		myListsWin = awin;
		myTitle = [atitle retain];
		[self setTitle: myTitle];
		
		songsArr = [[NSMutableArray alloc] initWithArray: songs];
		currSongPath = [[NSString alloc] init];
		
		myView = [[NSView alloc] initWithFrame: frame];
		[self setContentView: myView];
		
		buttsrect = NSMakeRect(0, frame.size.height -60, frame.size.width, 60);
		buttsFrame = [[ButtonsFrame alloc] initInWindow: self rect: buttsrect];
		[buttsFrame setAutoresizingMask: NSViewWidthSizable];
		[myView addSubview: buttsFrame];

		scrollrect = NSMakeRect(0, 0, frame.size.width, frame.size.height -60);
 		songsScroll = [[NSScrollView alloc] initWithFrame: scrollrect];
  		[songsScroll setHasVerticalScroller: YES];
  		[songsScroll setHasHorizontalScroller: YES];
		
  		buttonCell = [[NSButtonCell new] autorelease];
  		[buttonCell setButtonType: NSPushOnPushOffButton];
		[buttonCell setBordered:NO];
		[buttonCell setAlignment:NSLeftTextAlignment];	

 		songsMatrix = [[MPPlaySongsMatrix alloc] initWithFrame:scrollrect
				       								 	 			  mode:NSRadioModeMatrix
				  								  					prototype:buttonCell
			       						  					numberOfRows:0
			    						  					numberOfColumns:0
																	 inWindow:self];
											
		for(i = 0; i < [songsArr count]; i++) {
 			[songsMatrix addRow];
  			cell = [songsMatrix cellAtRow:i column:0];
			[cell setTitle: [[songsArr objectAtIndex:i] lastPathComponent]];
			[cell setTarget: self];
			[cell setAction: @selector(handleSongsMatrixSelect:)];
		}
  		[songsScroll setDocumentView: songsMatrix];
  		[myView addSubview: songsScroll];
		[self center];
	}
	return self;
}

- (void) dealloc
{
	[myTitle release];
	[songsArr release];
	[currSongPath release];
	[songsMatrix release];
	[songsScroll release];
  	[super dealloc];
}

- (BOOL)acceptsFirstResponder
{
	return YES;
}

- (BOOL)performKeyEquivalent:(NSEvent *)theEvent
{
	NSString *commchar;
	
	if(self == (MPPlaySongsWin *)[NSApp keyWindow]) {
		commchar = [theEvent charactersIgnoringModifiers];

		if([commchar isEqualToString: @"q"])
			[NSApp terminate: self];
		if([commchar isEqualToString: @"a"])
			[self addSong: nil];
		if([commchar isEqualToString: @"d"])
			[self removeSong: nil];

		return YES;
	}
	return NO;
}

- (NSString *)getTitle
{
	return myTitle;
}

- (NSArray *)songsArr;
{
	return songsArr;
}

- (void)handleSongsMatrixSelect:(id)sender
{
	NSString *cellTxt, *songPath;
	int i;
	
	cellTxt = [[songsMatrix selectedCell] title];
	for(i = 0; i < [songsArr count]; i++) {
		songPath = [songsArr objectAtIndex:i];
		if([[songPath lastPathComponent] isEqualToString: cellTxt]) {
			if(currSongPath)
				[currSongPath release];
			currSongPath = [[NSString alloc] initWithString: songPath];
		}
	}
}

- (void)addSong:(id)sender
{
	NSOpenPanel *panel;
	NSString *basePath, *songPath;
	id cell;
	BOOL found = NO;
	int i, result;
	
	panel = [NSOpenPanel openPanel];
	[panel setAllowsMultipleSelection:NO];
	[panel setCanChooseDirectories:NO];
	[panel setCanChooseFiles:YES];
	
	if([currSongPath cStringLength] > 2)
		basePath = [currSongPath stringByDeletingLastPathComponent];
	else
		basePath = [NSString stringWithString: NSHomeDirectory()];

	result = [panel runModalForDirectory: basePath file:@""];

	if(result == 1) {
		songPath = [[panel filenames] objectAtIndex: 0];
		if([[songPath pathExtension] isEqualToString: @"mp3"]) {
			for(i = 0; i < [songsArr count]; i++)
				if([[songsArr objectAtIndex:i] isEqualToString: songPath]) 
					found = YES;
			if(found) {
				[panel close];
				[self runAlert:@"This song is already present!"];
				return;
			} else {
				[songsArr addObject: songPath];
				[songsMatrix addRow];
  				cell = [songsMatrix cellAtRow:[songsArr count]-1 column:0];
				[cell setTitle: [songPath lastPathComponent]];
				[cell setTarget: self];
				[cell setAction: @selector(handleSongsMatrixSelect:)];
				[panel close];
				
				[myListsWin saveChangesInSongsWin: self];
			}
		}
	}
}

- (void)addSongFromDnD:(NSString *)songPath
{
	id cell;
	BOOL found = NO;
	int i;
	
	if(![[songPath pathExtension] isEqualToString: @"mp3"]) {
		[self runAlert:@"This is not a .mp3 file!"];
		return;
	}

	for(i = 0; i < [songsArr count]; i++)
		if([[songsArr objectAtIndex:i] isEqualToString: songPath]) 
			found = YES;
	if(found) {
		[self runAlert:@"This song is already present!"];
		return;
	} else {
		[songsArr addObject: songPath];
		[songsMatrix addRow];
  		cell = [songsMatrix cellAtRow:[songsArr count]-1 column:0];
		[cell setTitle: [songPath lastPathComponent]];
		[cell setTarget: self];
		[cell setAction: @selector(handleSongsMatrixSelect:)];

		[myListsWin saveChangesInSongsWin: self];
	}
}

- (void)removeSong:(id)sender
{
	NSString *cellTxt, *songPath;
	id cell;
	float x, y;
	int i, result;
	
	cellTxt = [[songsMatrix selectedCell] title];
	result = NSRunAlertPanel(@"delete song", 
[NSString stringWithFormat:@"Are you sure you want to delete
\"%@\"?", cellTxt], @"OK", @"Cancel", NULL);
	if(result != NSAlertDefaultReturn)
		return;
	
	for(i = 0; i < [songsArr count]; i++) {
		songPath = [songsArr objectAtIndex:i];
		if([[songPath lastPathComponent] isEqualToString: cellTxt]) {
			[songsArr removeObject: songPath];
			break;
		}
	}

	x = [songsMatrix frame].size.width;
	y = [songsMatrix frame].size.height;
	[songsMatrix release];
  	songsMatrix = [[NSMatrix alloc] initWithFrame:NSMakeRect(0, 0, x, y)
				       								  mode:NSRadioModeMatrix
				  								  	prototype:buttonCell
			       						  	numberOfRows:0
			    						  	numberOfColumns:0];
	for(i = 0; i < [songsArr count]; i++) {
 		[songsMatrix addRow];
  		cell = [songsMatrix cellAtRow:i column:0];
		[cell setTitle: [[songsArr objectAtIndex:i] lastPathComponent]];
		[cell setTarget: self];
		[cell setAction: @selector(handleSongsMatrixSelect:)];
	}
  	[songsScroll setDocumentView: songsMatrix];
  	[songsMatrix setNeedsDisplay: YES];

	[myListsWin saveChangesInSongsWin: self];
}

- (void)playSong:(id)sender
{
	[[MPApp sharedmpapp] setNewSongToPlay: currSongPath];
	[[MPApp sharedmpapp] play];
}

- (void)playList:(id)sender
{
	[[MPApp sharedmpapp] setNewListToPlay: songsArr];
	[[MPApp sharedmpapp] play];
}

- (void)runAlert:(NSString *)error
{
	NSRunAlertPanel(nil, [NSString stringWithString:error], @"Continue", nil, nil);
}

@end
